from typing import Any, Dict, NoReturn

from chaosplt_experiment.storage import ExperimentService, ExecutionService

__all__ = ["MyExperimentStorage", "MyExecutionStorage"]


class MyExperimentStorage(ExperimentService):
    def __init__(self, config: Dict[str, Any]):
        self.some_flag = True

    def release(self) -> NoReturn:
        self.some_flag = False


class MyExecutionStorage(ExecutionService):
    def __init__(self, config: Dict[str, Any]):
        self.some_flag = True

    def release(self) -> NoReturn:
        self.some_flag = False



# -*- coding: utf-8 -*-
from uuid import UUID

from flask import abort, Blueprint, jsonify, request
from flask_login import current_user, login_required
from marshmallow import ValidationError

from chaosplt_experiment.schemas import upload_experiment_schema, \
    experiment_schema, execution_schema, executions_schema

__all__ = ["api"]

api = Blueprint("experiment", __name__)


@api.route('', methods=['POST'])
@login_required
def new():
    user_id = current_user.id
    try:
        payload = upload_experiment_schema.load(request.json)
    except ValidationError as err:
        return jsonify(err.messages), 422

    org_id = payload["org"]
    workspace_id = payload["workspace"]
    payload = payload["payload"]

    has_org = request.services.account.org.has_org_by_id(org_id=org_id)
    if not has_org:
        return jsonify({
            "message": "Organization is unknown"
        }), 422

    has_workspace = request.services.account.org.has_workspace_by_id(
        org_id, workspace_id=workspace_id)
    if not has_workspace:
        return jsonify({
            "message": "Workspace is unknown in this organization"
        }), 422

    experiment = request.services.experiment.create(
        user_id, org_id, workspace_id, payload)
    return experiment_schema.jsonify(experiment)


@api.route('<uuid:experiment_id>', methods=['GET'])
@login_required
def get(experiment_id: UUID):
    experiment = request.services.experiment.get(experiment_id)
    if not experiment:
        return abort(404)
    return experiment_schema.jsonify(experiment)


@api.route('<uuid:experiment_id>', methods=['DELETE'])
@login_required
def delete(experiment_id: UUID):
    request.services.experiment.delete(experiment_id)
    return "", 204


@api.route('<uuid:experiment_id>/executions', methods=['GET'])
@login_required
def get_executions(experiment_id: UUID):
    experiment = request.services.experiment.get(experiment_id)
    if not experiment:
        return jsonify({
            "message": "Experiment is unknown"
        }), 404
    return executions_schema.jsonify(experiment.executions)


@api.route('executions/<uuid:execution_id>', methods=['GET'])
@login_required
def get_execution(experiment_id: UUID, execution_id: UUID):
    execution = request.services.execution.get(execution_id)
    if not execution:
        return abort(404)
    return execution_schema.jsonify(execution)


@api.route('executions/<uuid:execution_id>', methods=['DELETE'])
@login_required
def delete_execution(experiment_id: UUID, execution_id: UUID):
    request.services.execution.delete(execution_id)
    return "", 204


@api.route('executions/<uuid:execution_id>/journal', methods=['GET'])
@login_required
def get_journal(experiment_id: UUID, execution_id: UUID):
    execution = request.services.execution.get(execution_id)
    if not execution:
        return abort(404)
    return jsonify(execution.payload)



from typing import Any, Dict, NoReturn, Tuple

from chaosplt_relational_storage import get_storage, \
    configure_storage, release_storage
import pkg_resources

from .concrete import ExperimentService, ExecutionService

__all__ = ["initialize_storage", "shutdown_storage", "ExperimentStorage",
           "ExecutionStorage"]


class ExecutionStorage(ExecutionService):
    def __init__(self, config: Dict[str, Any]):
        driver = get_storage(config)
        configure_storage(driver)
        ExecutionService.__init__(self, driver)

    def release(self) -> NoReturn:
        release_storage(self.driver)


class ExperimentStorage(ExperimentService):
    def __init__(self, config: Dict[str, Any]):
        driver = get_storage(config)
        configure_storage(driver)
        ExperimentService.__init__(self, driver)

    def release(self) -> NoReturn:
        release_storage(self.driver)


def initialize_storage(config: Dict[str, Any]) \
                       -> Tuple[ExperimentStorage, ExecutionStorage]:
    experiment_storage = None
    execution_storage = None
    for plugin in pkg_resources.iter_entry_points('chaosplatform.storage'):
        if plugin.name == "experiment":
            service_class = plugin.load()
            experiment_storage = service_class(config)

        if plugin.name == "execution":
            service_class = plugin.load()
            execution_storage = service_class(config)

    if not experiment_storage:
        experiment_storage = ExperimentStorage(config)

    if not execution_storage:
        execution_storage = ExecutionStorage(config)

    return experiment_storage, execution_storage


def shutdown_storage(experiment_storage: ExperimentStorage,
                     execution_storage: ExecutionStorage) -> NoReturn:
    experiment_storage.release()
    execution_storage.release()



from flask import Flask
from flask_marshmallow import Marshmallow
from marshmallow import fields

__all__ = ["ma", "upload_experiment_schema", "experiment_schema",
           "upload_execution_schema", "execution_schema", "executions_schema",
           "setup_schemas"]

ma = Marshmallow()


def setup_schemas(app: Flask):
    return ma.init_app(app)


class ExperimentSchema(ma.Schema):
    class Meta:
        ordered = True
    id = fields.UUID(required=True)
    user_id = fields.UUID(required=True)
    org_id = fields.UUID(required=True)
    workspace_id = fields.UUID(required=True)
    created_date = fields.DateTime()
    updated_date = fields.DateTime()
    payload = fields.Dict(keys=fields.String(), allow_none=True)
    url = ma.AbsoluteURLFor('experiment.get', experiment_id='<id>')
    links = ma.Hyperlinks({
        'self': ma.URLFor('experiment.get', experiment_id='<id>')
    })


class UploadExperimentSchema(ma.Schema):
    org = fields.UUID(required=True)
    workspace = fields.UUID(required=True)
    payload = fields.Dict(keys=fields.String(), required=True)


class ExecutionSchema(ma.Schema):
    class Meta:
        ordered = True
    id = fields.UUID(required=True)
    user_id = fields.UUID(required=True)
    org_id = fields.UUID(required=True)
    workspace_id = fields.UUID(required=True)
    experiment_id = fields.UUID(required=True)
    status = fields.String()
    payload = fields.Dict(keys=fields.String(), allow_none=True)
    url = ma.AbsoluteURLFor(
        'experiment.get_execution', experiment_id='<experiment_id>',
        execution_id='<id>')
    links = ma.Hyperlinks({
        'self': ma.URLFor(
            'experiment.get_execution', experiment_id='<experiment_id>',
            execution_id='<id>')
    })


class UploadExecutionSchema(ma.Schema):
    experiment_id = fields.UUID(required=True)
    journal = fields.Dict(keys=fields.String(), required=True)


upload_experiment_schema = UploadExperimentSchema()
experiment_schema = ExperimentSchema()

upload_execution_schema = UploadExecutionSchema()
execution_schema = ExecutionSchema()
executions_schema = ExecutionSchema(many=True)



from collections import OrderedDict
from typing import Any,     Dict
from uuid import UUID

from chaosplt_experiment.model import Execution, Experiment
from chaosplt_experiment.schemas import upload_experiment_schema, \
    experiment_schema, upload_execution_schema, execution_schema, \
    executions_schema
from flask import Flask
from marshmallow.exceptions import ValidationError
import pytest


def test_experiment_schema_requires_user_id(experiment_request: Dict[str, Any]):
    r = experiment_request.copy()
    del r["user_id"]
    
    with pytest.raises(ValidationError) as x:
        e = experiment_schema.load(r)
    assert "user_id" in x.value.messages


def test_experiment_schema_requires_org_id(experiment_request: Dict[str, Any]):
    r = experiment_request.copy()
    del r["org_id"]
    
    with pytest.raises(ValidationError) as x:
        e = experiment_schema.load(r)
    assert "org_id" in x.value.messages


def test_experiment_schema_requires_workspace_id(experiment_request: Dict[str, Any]):
    r = experiment_request.copy()
    del r["workspace_id"]
    
    with pytest.raises(ValidationError) as x:
        e = experiment_schema.load(r)
    assert "workspace_id" in x.value.messages


def test_experiment_schema(experiment_request: Dict[str, Any], user_id: UUID,
                           org_id: UUID, workspace_id: UUID, created_date: str,
                           updated_date: str, experiment: str):
    e = experiment_schema.load(experiment_request)
    assert e["user_id"] == user_id
    assert e["org_id"] == org_id
    assert e["workspace_id"] == workspace_id
    assert e["payload"] == experiment


def test_experiment_schema_no_payload(experiment_request: Dict[str, str],
                                      user_id: UUID, org_id: UUID,
                                      workspace_id: UUID, created_date: str,
                                      updated_date: str):
    r = experiment_request.copy()
    del r["payload"]

    e = experiment_schema.load(r)
    assert e["user_id"] == user_id
    assert e["org_id"] == org_id
    assert e["workspace_id"] == workspace_id
    assert "payload" not in e


def test_execution_schema(execution_request: Dict[str, str], user_id: UUID,
                          org_id: UUID, workspace_id: UUID,
                          experiment_id: UUID, journal: Dict[str, Any]):
    e = execution_schema.load(execution_request)
    assert e["user_id"] == user_id
    assert e["org_id"] == org_id
    assert e["workspace_id"] == workspace_id
    assert e["experiment_id"] == experiment_id
    assert e["payload"] == journal
    assert e["status"] == "completed"


def test_execution_schema_no_payload(execution_request: Dict[str, str],
                                     user_id: UUID, org_id: UUID,
                                     workspace_id: UUID, experiment_id: UUID,
                                     journal: Dict[str, Any]):
    r = execution_request.copy()
    del r["payload"]

    e = execution_schema.load(r)
    assert e["user_id"] == user_id
    assert e["org_id"] == org_id
    assert e["workspace_id"] == workspace_id
    assert e["experiment_id"] == experiment_id
    assert e["status"] == "completed"
    assert "payload" not in e


def test_execution_schema_requires_user_id(execution_request: Dict[str, Any]):
    r = execution_request.copy()
    del r["user_id"]
    
    with pytest.raises(ValidationError) as x:
        e = execution_schema.load(r)
    assert "user_id" in x.value.messages


def test_experiment_schema_requires_org_id(execution_request: Dict[str, Any]):
    r = execution_request.copy()
    del r["org_id"]
    
    with pytest.raises(ValidationError) as x:
        e = execution_schema.load(r)
    assert "org_id" in x.value.messages


def test_execution_schema_requires_workspace_id(execution_request: Dict[str, Any]):
    r = execution_request.copy()
    del r["workspace_id"]
    
    with pytest.raises(ValidationError) as x:
        e = execution_schema.load(r)
    assert "workspace_id" in x.value.messages


def test_execution_schema_requires_experiment_id(execution_request: Dict[str, Any]):
    r = execution_request.copy()
    del r["experiment_id"]
    
    with pytest.raises(ValidationError) as x:
        e = execution_schema.load(r)
    assert "experiment_id" in x.value.messages


def test_upload_experiment(org_id: UUID, workspace_id: UUID,
                           experiment: Dict[str, Any]):
    r = upload_experiment_schema.load({
        "org": org_id,
        "workspace": workspace_id,
        "payload": experiment
    })
    assert r["org"] == org_id
    assert r["workspace"] == workspace_id
    assert r["payload"] == experiment


def test_upload_experiment_requires_org(workspace_id: UUID,
                                        experiment: Dict[str, Any]):
    with pytest.raises(ValidationError) as x:
        upload_experiment_schema.load({
            "workspace": workspace_id,
            "payload": experiment
        })
    assert "org" in x.value.messages


def test_upload_experiment_requires_workspace(org_id: UUID,
                                              experiment: Dict[str, Any]):
    with pytest.raises(ValidationError) as x:
        upload_experiment_schema.load({
            "org": org_id,
            "payload": experiment
        })
    assert "workspace" in x.value.messages


def test_upload_experiment_requires_experiment(org_id: UUID, workspace_id: UUID):
    with pytest.raises(ValidationError) as x:
        upload_experiment_schema.load({
            "workspace": workspace_id,
            "org": org_id
        })
    assert "payload" in x.value.messages


def test_upload_execution_requires_experiment_id(journal: Dict[str, Any]):
    with pytest.raises(ValidationError) as x:
        upload_execution_schema.load({
            "payload": journal
        })
    assert "experiment_id" in x.value.messages


def test_upload_execution_requires_journal(experiment_id: UUID):
    with pytest.raises(ValidationError) as x:
        upload_execution_schema.load({
            "experiment_id": experiment_id
        })
    assert "journal" in x.value.messages



import os
import sys

from pkg_resources import Distribution, EntryPoint, iter_entry_points, \
    working_set

from chaosplt_experiment.storage import initialize_storage, shutdown_storage


def test_loading_external_storage_implementation():
    try:
        distribution = Distribution(os.path.join(
                os.path.dirname(__file__), "fixtures"))
        experiment_entry_point = EntryPoint.parse(
            'experiment = fake_storage:MyExperimentStorage', dist=distribution)
        execution_entry_point = EntryPoint.parse(
            'execution = fake_storage:MyExecutionStorage', dist=distribution)
        distribution._ep_map = {
            'chaosplatform.storage': {
                'experiment': experiment_entry_point,
                'execution': execution_entry_point
            }
        }
        working_set.add(distribution)

        exp_storage, exec_storage = initialize_storage(
            config={"db":{"uri": "sqlite:///"}})
        assert exp_storage.__class__.__name__ == "MyExperimentStorage"
        assert exec_storage.__class__.__name__ == "MyExecutionStorage"
        assert exp_storage.some_flag == True
        assert exec_storage.some_flag == True

        shutdown_storage(exp_storage, exec_storage)
        assert exp_storage.some_flag == False
        assert exec_storage.some_flag == False

    finally:
        distribution._ep_map.clear()



from datetime import datetime
import logging
from logging import StreamHandler
import os
from typing import Any, Dict, Tuple
from unittest.mock import patch, MagicMock
import uuid
from uuid import UUID

import cherrypy
from flask import Flask, request
from flask_caching import Cache
import pytest
import simplejson as json

from chaosplt_experiment.cache import setup_cache
from chaosplt_experiment.log import http_requests_logger
from chaosplt_experiment.views.web import create_app, serve_experiment_app, \
    serve_execution_app
from chaosplt_experiment.views.api import create_api, serve_experiment_api, \
    serve_execution_api
from chaosplt_experiment.service import Services, initialize_services
from chaosplt_experiment.settings import load_settings
from chaosplt_experiment.storage import ExperimentStorage, ExecutionStorage, \
    initialize_storage

cur_dir = os.path.abspath(os.path.dirname(__file__))
fixtures_dir = os.path.join(cur_dir, "fixtures")
env_path = os.path.join(fixtures_dir, '.env')


@pytest.fixture
def config() -> Dict[str, Any]:
    return load_settings(env_path)


@pytest.fixture
def storages(config: Dict[str, Any]) \
             -> Tuple[ExperimentStorage, ExecutionStorage]:
    return initialize_storage(config)


@pytest.fixture
def experiment_storage(storages: Tuple[ExperimentStorage, ExecutionStorage]) \
                       -> ExperimentStorage:
    return storages[0]


@pytest.fixture
def execution_storage(storages: Tuple[ExperimentStorage, ExecutionStorage]) \
                      -> ExecutionStorage:
    return storages[1]


@pytest.fixture
def stream_handler() -> StreamHandler:
    return StreamHandler()


@pytest.fixture
def services(experiment_storage: ExperimentStorage,
             execution_storage: ExecutionStorage,
             config: Dict[str, Any]) -> Services:
    services = Services()
    services.experiment = experiment_storage
    services.execution = execution_storage
    initialize_services(services, config)
    return services


@pytest.fixture
def app(config: Dict[str, Any], services: Services,
        experiment_storage: ExperimentStorage,
        execution_storage: ExecutionStorage,
        stream_handler: StreamHandler) -> Flask:
    application = create_app(config)
    cache = setup_cache(application)
    serve_experiment_app(
        application, cache, services, experiment_storage, config,
        "/experiments", log_handler=stream_handler)
    serve_execution_app(
        application, cache, services, execution_storage, config,
        "/executions", log_handler=stream_handler)
    return application


@pytest.fixture
def api(app: Flask, config: Dict[str, Any], services: Services,
        experiment_storage: ExperimentStorage,
        execution_storage: ExecutionStorage,
        stream_handler: StreamHandler) -> Flask:
    application = create_api(config)
    cache = setup_cache(application)
    wsgiapp = http_requests_logger(application, stream_handler)
    cherrypy.tree.graft(wsgiapp, "/api/v1")
    serve_experiment_api(
        application, cache, services, experiment_storage, config,
        "/api/v1/experiments", log_handler=stream_handler)
    serve_execution_api(
        application, cache, services, execution_storage, config,
        "/api/v1/executions", log_handler=stream_handler)
    return application


@pytest.fixture
def app_cache(app: Flask) -> Cache:
    return Cache(app, config=app.config)


@pytest.fixture
def api_cache(api: Flask) -> Cache:
    return Cache(api, config=api.config)


@pytest.fixture
def user_id() -> UUID:
    return uuid.uuid4()


@pytest.fixture
def org_id() -> UUID:
    return uuid.uuid4()


@pytest.fixture
def workspace_id() -> UUID:
    return uuid.uuid4()


@pytest.fixture
def experiment_id() -> UUID:
    return uuid.uuid4()


@pytest.fixture
def created_date() -> str:
    return datetime.utcnow().isoformat()


@pytest.fixture
def updated_date() -> str:
    return datetime.utcnow().isoformat()


@pytest.fixture
def experiment() -> Dict[str, Any]:
    return {
        "title": "hello"
    }


@pytest.fixture
def experiment_request(user_id: UUID, org_id: UUID, workspace_id: UUID,
                       created_date: str, updated_date: str,
                       experiment: str) -> Dict[str, Any]:
    return {
        "id": uuid.uuid4(),
        "user_id": user_id,
        "org_id": org_id,
        "workspace_id": workspace_id,
        "created_date": created_date,
        "updated_date": updated_date,
        "payload": experiment
    }


@pytest.fixture
def journal() -> Dict[str, Any]:
    return {
        "status": "completed"
    }


@pytest.fixture
def execution_request(user_id: UUID, org_id: UUID, workspace_id: UUID,
                      experiment_id: UUID, journal: str) -> Dict[str, Any]:
    return {
        "id": uuid.uuid4(),
        "user_id": user_id,
        "org_id": org_id,
        "workspace_id": workspace_id,
        "experiment_id": experiment_id,
        "status": "completed",
        "payload": journal
    }


from typing import Any, Dict
from unittest.mock import ANY, MagicMock, patch

from chaosplt_experiment.service import Services
from chaosplt_experiment.server import initialize_all, release_all


@patch('chaosplt_experiment.server.create_grpc_server', autospec=True)
def test_initialize_all(create_grpc_server, config: Dict[str, Any],
                        services: Services):
    grpc_server = MagicMock()
    create_grpc_server.return_value = grpc_server

    try:
        web_app, api_app, services, grpc_server, experiment_storage, \
            execution_storage = initialize_all(config, services, None)

        assert services.experiment is not None
        assert services.execution is not None
    finally:
        release_all(
            web_app, api_app, services, grpc_server, experiment_storage,
            execution_storage)



from datetime import datetime
from typing import Any, Dict, List
from uuid import UUID

import attr

__all__ = ["Experiment", "Execution"]


@attr.s
class Execution:
    id: UUID = attr.ib()
    user_id: UUID = attr.ib()
    org_id: UUID = attr.ib()
    workspace_id: UUID = attr.ib()
    experiment_id: UUID = attr.ib()
    payload: Dict[str, Any] = attr.ib(default=None)


@attr.s
class Experiment:
    id: UUID = attr.ib()
    user_id: UUID = attr.ib()
    org_id: UUID = attr.ib()
    workspace_id: UUID = attr.ib()
    created_date: datetime = attr.ib()
    updated_date: datetime = attr.ib()
    payload: Dict[str, Any] = attr.ib(default=None)
    executions: List[Execution] = attr.ib(default=None)



import logging
import os
from logging import StreamHandler
from typing import Any, Dict
from unittest.mock import MagicMock

from requestlogger import ApacheFormatter, WSGILogger

from chaosplt_experiment.log import clean_logger, configure_logger, \
    http_requests_logger


def test_configure_logger_with_embedded_config(config: Dict[str, Any]):
    logger = logging.getLogger("chaosplatform")
    assert len(logger.handlers) == 0

    try:
        configure_logger(None, config)
        assert logger.propagate is False
        assert logger.level == logging.INFO
        assert len(logger.handlers) == 1
        h = logger.handlers[0]
        assert isinstance(h, StreamHandler)
        assert h.level == logging.INFO
    finally:
        clean_logger()



def test_override_log_level(config: Dict[str, Any]):
    config = config.copy()
    config["debug"] = True

    logger = logging.getLogger("chaosplatform")
    assert len(logger.handlers) == 0

    try:
        configure_logger(None, config)
        assert logger.propagate is False
        assert logger.level == logging.DEBUG

        assert len(logger.handlers) == 1
        h = logger.handlers[0]
        assert isinstance(h, StreamHandler)
        assert h.level == logging.DEBUG
    finally:
        clean_logger()


def test_configure_logger_with_specific_config(config: Dict[str, Any]):
    logger = logging.getLogger("chaosplatform")
    assert len(logger.handlers) == 0

    try:
        p = os.path.join(
            os.path.dirname(__file__), "fixtures", "dummy-logging.json")
        configure_logger(p, config)
        assert logger.propagate is False
        assert logger.level == logging.ERROR
        assert len(logger.handlers) == 1
        h = logger.handlers[0]
        assert isinstance(h, StreamHandler)
        assert h.level == logging.ERROR
    finally:
        clean_logger()


def test_http_requests_logger():
    app = MagicMock()
    wsgi_logger = http_requests_logger(app)
    assert isinstance(wsgi_logger, WSGILogger)


import pytest

from chaosplt_experiment.storage.interface import BaseExperimentService, \
    BaseExecutionService


def test_cannot_instanciate_experiment_service_interface():
    try:
        BaseExperimentService()
    except TypeError as e:
        return
    else:
        pytest.fail("BaseExperimentService should remain abstract")


def test_cannot_instanciate_execution_service_interface():
    try:
        BaseExecutionService()
    except TypeError as e:
        return
    else:
        pytest.fail("BaseExecutionService should remain abstract")



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Any, List, NoReturn, Union
import uuid
from uuid import UUID

from chaosplt_relational_storage.db import Base
from sqlalchemy import Column, DateTime, func
from sqlalchemy.orm import relationship, defer
from sqlalchemy.orm.session import Session
from sqlalchemy_json import NestedMutable
from sqlalchemy_utils import UUIDType
from sqlalchemy_utils import JSONType as JSONB

__all__ = ["Experiment"]


class Experiment(Base):  # type: ignore
    __tablename__ = 'experiment'
    id = Column(
        UUIDType(binary=False), primary_key=True, default=uuid.uuid4)
    user_id = Column(UUIDType(binary=False), nullable=False, index=True)
    org_id = Column(
        UUIDType(binary=False), nullable=False, index=True)
    workspace_id = Column(
        UUIDType(binary=False), nullable=False, index=True)
    created_date = Column(DateTime(), default=datetime.utcnow)
    updated_date = Column(
        DateTime(), default=datetime.utcnow, server_onupdate=func.now())
    suggested_experiment_id = Column(UUIDType(binary=False), index=True)
    executions = relationship(
        'Execution', backref='experiment', cascade="all, delete-orphan")
    payload = Column(NestedMutable.as_mutable(JSONB), nullable=False)

    @staticmethod
    def load(experiment_id: Union[UUID, str],
             session: Session) -> 'Experiment':
        return session.query(Experiment).filter_by(id=experiment_id).first()

    @staticmethod
    def load_by_user(user_id: Union[UUID, str],
                     experiment_id: Union[UUID, str],
                     session: Session) -> 'Experiment':
        return session.query(Experiment).\
            filter_by(id=experiment_id).\
            filter_by(user_id=user_id).first()

    @staticmethod
    def load_by_workspace(workspace_id: Union[UUID, str],
                          session: Session) -> List['Experiment']:
        return session.query(Experiment).filter_by(
            workspace_id=workspace_id).all()

    @staticmethod
    def list_all_by_user(user_id: Union[UUID, str],
                         session: Session) -> List['Experiment']:
        return session.query(Experiment).options(defer('payload')).filter_by(
            user_id=user_id).all()

    @staticmethod
    def create(user_id: Union[UUID, str], org_id: Union[UUID, str],
               workspace_id: Union[UUID, str], experiment: Any,
               session: Session) -> 'Experiment':
        experiment = Experiment(
            user_id=user_id, org_id=org_id, workspace_id=workspace_id,
            payload=experiment)
        session.add(experiment)
        return experiment

    @staticmethod
    def delete(experiment_id: Union[UUID, str],
               session: Session) -> NoReturn:
        experiment = session.query(Experiment).filter_by(
            id=experiment_id).first()

        if experiment:
            session.delete(experiment)



# -*- coding: utf-8 -*-
import uuid

from chaosplt_relational_storage.db import Base
from sqlalchemy import Column, DateTime, Float, String
from sqlalchemy_utils import UUIDType
from sqlalchemy_utils import JSONType as JSONB

__all__ = ["Recommendation"]


class Recommendation(Base):  # type: ignore
    __tablename__ = 'experiment_recommendation'
    id = Column(
        UUIDType(binary=False), primary_key=True, default=uuid.uuid4)
    recommendation_type = Column(String(), index=True)
    checksum = Column(String(), nullable=False)
    last_updated = Column(DateTime())
    rating = Column(Float, default=3.0)
    meta = Column(JSONB())
    data = Column(JSONB(), nullable=False)



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Any, List, NoReturn, Union
import uuid
from uuid import UUID

from chaosplt_relational_storage.db import Base
from sqlalchemy import Column, ForeignKey, String, UniqueConstraint, BigInteger
from sqlalchemy.orm import defer
from sqlalchemy.orm.session import Session
from sqlalchemy_utils import UUIDType
from sqlalchemy_utils import JSONType as JSONB

__all__ = ["Execution"]


class Execution(Base):  # type: ignore
    __tablename__ = 'experiment_execution'
    __table_args__ = (
        UniqueConstraint(
            'timestamp', 'experiment_id',
        ),
    )

    id = Column(
        UUIDType(binary=False), primary_key=True, default=uuid.uuid4)
    user_id = Column(UUIDType(binary=False), nullable=False, index=True)
    timestamp = Column(
        BigInteger,
        default=lambda: int(datetime.utcnow().timestamp() * 1000))
    org_id = Column(UUIDType(binary=False), nullable=False, index=True)
    workspace_id = Column(
        UUIDType(binary=False), nullable=False, index=True)
    experiment_id = Column(
        UUIDType(binary=False), ForeignKey(
            'experiment.id', ondelete='CASCADE'), nullable=False)
    status = Column(String)
    payload = Column(JSONB())

    @staticmethod
    def load(execution_id: Union[UUID, str],
             session: Session) -> 'Execution':
        return session.query(Execution).filter_by(id=execution_id).first()

    @staticmethod
    def load_by_user(user_id: Union[UUID, str],
                     session: Session) -> List['Experiment']:
        return session.query(Execution).options(defer('payload')).filter_by(
            user_id=user_id).all()

    @staticmethod
    def create(user_id: Union[UUID, str], org_id: Union[UUID, str],
               workspace_id: Union[UUID, str], experiment_id: Union[UUID, str],
               execution: Any, session: Session) -> 'Execution':
        execution = Execution(
            user_id=user_id,
            org_id=org_id,
            workspace_id=workspace_id,
            experiment_id=experiment_id,
            payload=execution
        )
        session.add(execution)
        return execution

    @staticmethod
    def delete(execution_id: Union[UUID, str], session: Session) -> NoReturn:
        execution = session.query(Execution).filter_by(
            id=execution_id).first()

        if execution:
            session.delete(execution)



from abc import ABC, abstractmethod
from uuid import UUID
from typing import Any, List, NoReturn, Union

from chaosplt_experiment.model import Experiment, Execution

__all__ = ["BaseExperimentService", "BaseExecutionService"]


class BaseExperimentService(ABC):
    @abstractmethod
    def get(self, experiment_id: Union[UUID, str]) -> Experiment:
        raise NotImplementedError()

    @abstractmethod
    def create(self, user_id: Union[UUID, str], org_id: Union[UUID, str],
               workspace_id: Union[UUID, str], experiment: Any) -> Experiment:
        return self.save(user_id, org_id, workspace_id, experiment)

    @abstractmethod
    def delete(self, experiment_id: Union[UUID, str]) -> NoReturn:
        raise NotImplementedError()

    @abstractmethod
    def has_experiment_by_id(self, experiment_id: Union[UUID, str]) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def get_by_workspace(self, workspace_id: Union[UUID, str]) \
                         -> List[Experiment]:  # noqa: E127
        raise NotImplementedError()

    @abstractmethod
    def list_by_user(self, user_id: Union[UUID, str]) -> List[Experiment]:
        raise NotImplementedError()

    @abstractmethod
    def get_by_user(self, user_id: Union[UUID, str],
                    experiment_id: Union[UUID, str]) -> Experiment:
        raise NotImplementedError()


class BaseExecutionService(ABC):
    @abstractmethod
    def get(self, execution_id: Union[UUID, str]) -> Execution:
        raise NotImplementedError()

    @abstractmethod
    def create(self, experiment_id: Union[UUID, str],
               execution: Any) -> Execution:
        raise NotImplementedError()

    @abstractmethod
    def delete(self, execution_id: Union[UUID, str]) -> NoReturn:
        raise NotImplementedError()

    @abstractmethod
    def get_by_user(self, user_id: Union[UUID, str]) -> List[Execution]:
        raise NotImplementedError()



from typing import Any, Dict, List, NoReturn, Union
from uuid import UUID

from chaosplt_experiment.model import Experiment, Execution
from chaosplt_relational_storage import RelationalStorage
from chaosplt_relational_storage.db import orm_session
from .model import Experiment as ExperimentModel, Execution as ExecutionModel

from .interface import BaseExperimentService, BaseExecutionService

__all__ = ["ExperimentService", "ExecutionService"]


class ExperimentService(BaseExperimentService):
    def __init__(self, driver: RelationalStorage):
        self.driver = driver

    def get(self, experiment_id: Union[UUID, str]) -> Experiment:
        with orm_session() as session:
            experiment = ExperimentModel.load(experiment_id, session=session)
            if not experiment:
                return

            return Experiment(
                id=experiment.id,
                user_id=experiment.user_id,
                org_id=experiment.org_id,
                workspace_id=experiment.workspace_id,
                created_date=experiment.created_date,
                updated_date=experiment.updated_date,
                payload=experiment.payload,
                executions=[
                    Execution(
                        id=execution.id,
                        user_id=experiment.user_id,
                        org_id=experiment.org_id,
                        workspace_id=experiment.workspace_id,
                        experiment_id=experiment.id
                    ) for execution in experiment.executions
                ] if experiment.executions else None
            )

    def get_chaosplatform_extension(self, experiment: Any) -> Dict[str, Any]:
        extensions = experiment.setdefault("extensions", [])
        for x in extensions:
            if x.get("name") == "chaosplatform":
                return x
        extension = {"name": "chaosplatform"}
        extensions.append(extension)
        return extension

    def create(self, user_id: Union[UUID, str], org_id: Union[UUID, str],
               workspace_id: Union[UUID, str], experiment: Any) -> Experiment:
        with orm_session() as session:
            payload = experiment
            extension = self.get_chaosplatform_extension(experiment)
            experiment = ExperimentModel.create(
                user_id, org_id, workspace_id=workspace_id,
                experiment=experiment, session=session)
            session.flush()

            extension.setdefault("experiment_id", str(experiment.id))
            extension["org_id"] = str(org_id)
            extension["workspace_id"] = str(workspace_id)
            experiment.payload = payload

            return Experiment(
                id=experiment.id,
                user_id=experiment.user_id,
                org_id=experiment.org_id,
                workspace_id=experiment.workspace_id,
                created_date=experiment.created_date,
                updated_date=experiment.updated_date,
                payload=experiment.payload
            )

    def delete(self, experiment_id: Union[UUID, str]) -> NoReturn:
        with orm_session() as session:
            ExperimentModel.delete(experiment_id, session=session)

    def get_by_workspace(self, workspace_id: Union[UUID, str]) \
                         -> List[Experiment]:  # noqa: E127
        with orm_session() as session:
            experiments = []

            candidates = ExperimentModel.load_by_workspace(
                workspace_id, session=session)
            for experiment in candidates:
                experiments.append(
                    Experiment(
                        id=experiment.id,
                        user_id=experiment.user_id,
                        org_id=experiment.org_id,
                        workspace_id=experiment.workspace_id,
                        created_date=experiment.created_date,
                        updated_date=experiment.updated_date
                    )
                )
            return experiments

    def get_by_user(self, user_id: Union[UUID, str],
                    experiment_id: Union[UUID, str]) -> Experiment:
        with orm_session() as session:
            experiment = ExperimentModel.load_by_user(
                user_id, experiment_id, session=session)
            if not experiment:
                return

            return Experiment(
                id=experiment.id,
                user_id=experiment.user_id,
                org_id=experiment.org_id,
                workspace_id=experiment.workspace_id,
                created_date=experiment.created_date,
                updated_date=experiment.updated_date,
                payload=experiment.payload,
                executions=[
                    Execution(
                        id=execution.id,
                        user_id=experiment.user_id,
                        org_id=experiment.org_id,
                        workspace_id=experiment.workspace_id,
                        experiment_id=experiment.id
                    ) for execution in experiment.executions
                ] if experiment.executions else None
            )

    def list_by_user(self, user_id: Union[UUID, str]) -> List[Experiment]:
        with orm_session() as session:
            experiments = []

            candidates = ExperimentModel.list_all_by_user(
                user_id, session=session)
            for experiment in candidates:
                experiments.append(
                    Experiment(
                        id=experiment.id,
                        user_id=experiment.user_id,
                        org_id=experiment.org_id,
                        workspace_id=experiment.workspace_id,
                        created_date=experiment.created_date,
                        updated_date=experiment.updated_date
                    )
                )
            return experiments

    def has_experiment_by_id(self, experiment_id: Union[UUID, str]) -> bool:
        with orm_session() as session:
            experiment = ExperimentModel.load(experiment_id, session=session)
            return experiment is not None


class ExecutionService(BaseExecutionService):
    def __init__(self, driver: RelationalStorage):
        self.driver = driver

    def get(self, execution_id: Union[UUID, str]) -> Execution:
        with orm_session() as session:
            execution = ExecutionModel.load(execution_id, session=session)
            if not execution:
                return

            return Execution(
                id=execution.id,
                user_id=execution.user_id,
                org_id=execution.org_id,
                workspace_id=execution.workspace_id,
                experiment_id=execution.experiment_id,
                payload=execution.payload
            )

    def create(self, experiment_id: Union[UUID, str],
               execution: Any) -> Execution:
        with orm_session() as session:
            experiment = ExperimentModel.load(experiment_id, session=session)
            execution = ExecutionModel.create(
                experiment.user_id, experiment.org_id, experiment.workspace_id,
                experiment.id, execution=execution, session=session)
            session.flush()

            return Execution(
                id=execution.id,
                user_id=execution.user_id,
                org_id=execution.org_id,
                workspace_id=execution.workspace_id,
                experiment_id=execution.experiment_id,
                payload=execution.payload
            )

    def delete(self, execution_id: Union[UUID, str]) -> NoReturn:
        with orm_session() as session:
            ExecutionModel.delete(execution_id, session=session)

    def get_by_user(self, user_id: Union[UUID, str]) -> List[Execution]:
        with orm_session() as session:
            executions = []

            candidates = ExecutionModel.load_by_user(
                user_id, session=session)
            for execution in candidates:
                executions.append(
                    Execution(
                        id=execution.id,
                        user_id=execution.user_id,
                        org_id=execution.org_id,
                        workspace_id=execution.workspace_id,
                        experiment_id=execution.experiment_id,
                        payload=execution.payload
                    )
                )
            return executions



import attr
import uuid

from chaosplt_experiment.storage import ExperimentStorage, ExecutionStorage
from chaosplt_experiment.storage.concrete import ExperimentService
from chaosplt_relational_storage.db import orm_session


def test_create_experiment(experiment_storage: ExperimentStorage):
    e = experiment_storage.create(
        user_id=uuid.uuid4(),
        org_id=uuid.uuid4(),
        workspace_id=uuid.uuid4(),
        experiment={"title": "a tale of one platform"}
    )
    assert uuid.UUID(hex=e.id.hex) == e.id


def test_load_experiment(experiment_storage: ExperimentStorage):
    e = experiment_storage.create(
        user_id=uuid.uuid4(),
        org_id=uuid.uuid4(),
        workspace_id=uuid.uuid4(),
        experiment={"title": "a tale of one platform"}
    )

    fetched_exp = experiment_storage.get(e.id)
    assert e == fetched_exp


def test_load_experiment_by_user(experiment_storage: ExperimentStorage):
    user_id = uuid.uuid4()

    e = experiment_storage.create(
        user_id=user_id,
        org_id=uuid.uuid4(),
        workspace_id=uuid.uuid4(),
        experiment={"title": "a tale of one platform"}
    )

    fetched_exp = experiment_storage.get_by_user(user_id, e.id)
    assert fetched_exp == e

    fetched_exp = experiment_storage.get_by_user(uuid.uuid4(), e.id)
    assert fetched_exp is None


def test_load_experiment_list_by_user(experiment_storage: ExperimentStorage):
    user_id = uuid.uuid4()

    e = experiment_storage.create(
        user_id=user_id,
        org_id=uuid.uuid4(),
        workspace_id=uuid.uuid4(),
        experiment={"title": "a tale of one platform"}
    )
    fetched_exps = experiment_storage.list_by_user(user_id)
    assert fetched_exps[0].id == e.id
    assert fetched_exps[0].user_id == e.user_id
    assert fetched_exps[0].org_id == e.org_id
    assert fetched_exps[0].workspace_id == e.workspace_id
    assert fetched_exps[0].created_date == e.created_date
    assert fetched_exps[0].updated_date == e.updated_date
    assert fetched_exps[0].payload is None
    assert fetched_exps[0].executions is None

    fetched_exps = experiment_storage.list_by_user(uuid.uuid4())
    assert fetched_exps == []


def test_load_experiment_by_workspace(experiment_storage: ExperimentStorage):
    user_id = uuid.uuid4()
    workspace_id = uuid.uuid4()

    e = experiment_storage.create(
        user_id=user_id,
        org_id=uuid.uuid4(),
        workspace_id=workspace_id,
        experiment={"title": "a tale of one platform"}
    )

    fetched_exps = experiment_storage.get_by_workspace(workspace_id)
    assert fetched_exps[0].id == e.id
    assert fetched_exps[0].user_id == e.user_id
    assert fetched_exps[0].org_id == e.org_id
    assert fetched_exps[0].workspace_id == e.workspace_id
    assert fetched_exps[0].created_date == e.created_date
    assert fetched_exps[0].updated_date == e.updated_date
    assert fetched_exps[0].payload is None
    assert fetched_exps[0].executions is None

    fetched_exp = experiment_storage.get_by_workspace(uuid.uuid4())
    assert fetched_exp == []


def test_delete_experiment(experiment_storage: ExperimentStorage):
    user_id = uuid.uuid4()
    workspace_id = uuid.uuid4()

    e = experiment_storage.create(
        user_id=user_id,
        org_id=uuid.uuid4(),
        workspace_id=workspace_id,
        experiment={"title": "a tale of one platform"}
    )

    fetched_exp = experiment_storage.get(e.id)
    assert e == fetched_exp

    experiment_storage.delete(e.id)
    fetched_exp = experiment_storage.get(e.id)
    assert fetched_exp == None


def test_create_execution(experiment_storage: ExperimentStorage,
                          execution_storage: ExecutionStorage):
    e = experiment_storage.create(
        user_id=uuid.uuid4(),
        org_id=uuid.uuid4(),
        workspace_id=uuid.uuid4(),
        experiment={"title": "a tale of one platform"}
    )
    e = execution_storage.create(
        experiment_id=e.id,
        execution={}
    )
    assert uuid.UUID(hex=e.id.hex) == e.id


def test_load_execution(experiment_storage: ExperimentStorage,
                        execution_storage: ExecutionStorage):
    e = experiment_storage.create(
        user_id=uuid.uuid4(),
        org_id=uuid.uuid4(),
        workspace_id=uuid.uuid4(),
        experiment={"title": "a tale of one platform"}
    )
    e = execution_storage.create(
        experiment_id=e.id,
        execution={}
    )

    fetched_exp = execution_storage.get(e.id)
    assert e == fetched_exp


def test_load_execution_by_user(experiment_storage: ExperimentStorage,
                                execution_storage: ExecutionStorage):
    user_id = uuid.uuid4()
    e = experiment_storage.create(
        user_id=user_id,
        org_id=uuid.uuid4(),
        workspace_id=uuid.uuid4(),
        experiment={"title": "a tale of one platform"}
    )
    e = execution_storage.create(
        experiment_id=e.id,
        execution={}
    )

    fetched_exs = execution_storage.get_by_user(user_id)
    assert fetched_exs == [e]

    fetched_exs = execution_storage.get_by_user(uuid.uuid4())
    assert fetched_exs == []



def test_delete_execution(experiment_storage: ExperimentStorage,
                          execution_storage: ExecutionStorage):
    user_id = uuid.uuid4()
    e = experiment_storage.create(
        user_id=user_id,
        org_id=uuid.uuid4(),
        workspace_id=uuid.uuid4(),
        experiment={"title": "a tale of one platform"}
    )
    e = execution_storage.create(
        experiment_id=e.id,
        execution={}
    )


    fetched_exp = execution_storage.get(e.id)
    assert e == fetched_exp

    execution_storage.delete(e.id)
    fetched_ex = execution_storage.get(e.id)
    assert fetched_ex == None



from typing import Any, Dict

import attr

__all__ = ["initialize_services", "shutdown_services", "Services"]


@attr.s
class Services:
    experiment: object = attr.ib(default=None)
    execution: object = attr.ib(default=None)


def initialize_services(services: Services, config: Dict[str, Any]):
    pass


def shutdown_services(services: Services):
    pass



# -*- coding: utf-8 -*-
import logging
from logging import StreamHandler
import os
from typing import Any, Dict, Union

from flask import Blueprint, Flask, after_this_request, request, Response
from flask_caching import Cache

from chaosplt_experiment.auth import setup_jwt
from chaosplt_experiment.schemas import setup_schemas
from chaosplt_experiment.service import Services
from chaosplt_experiment.storage import ExperimentStorage, ExecutionStorage

__all__ = ["cleanup_app", "create_app", "serve_experiment_app",
           "serve_execution_app"]


def create_app(config: Dict[str, Any]) -> Flask:
    app = Flask(__name__)

    app.url_map.strict_slashes = False
    app.debug = True if os.getenv('CHAOSHUB_DEBUG') else False

    logger = logging.getLogger('flask.app')
    logger.propagate = False

    app.config["SECRET_KEY"] = os.getenv("SECRET_KEY")
    app.config["SQLALCHEMY_DATABASE_URI"] = os.getenv("DATABASE_URI")
    app.config["JWT_SECRET_KEY"] = os.getenv("JWT_SECRET_KEY")
    app.secret_key = app.config["SECRET_KEY"]

    app.config["GRPC_ADDR"] = "{}:{}".format(
        os.getenv("GRPC_LISTEN_ADDR"), os.getenv("GRPC_LISTEN_PORT"))

    app.config["CACHE_TYPE"] = os.getenv("CACHE_TYPE", "simple")
    if app.config["CACHE_TYPE"] == "redis":
        app.config["CACHE_REDIS_HOST"] = os.getenv("CACHE_REDIS_HOST")
        app.config["CACHE_REDIS_PORT"] = os.getenv("CACHE_REDIS_PORT", 6379)

    setup_jwt(app)
    setup_schemas(app)

    return app


def cleanup_app(app: Flask):
    pass


def serve_experiment_app(app: Flask, cache: Cache, services: Services,
                         storage: ExperimentStorage, config: Dict[str, Any],
                         mount_point: str = '/experiment',
                         log_handler: StreamHandler = None):
    register_experiment_views(app, cache, services, storage)


def serve_execution_app(app: Flask, cache: Cache, services: Services,
                        storage: ExperimentStorage, config: Dict[str, Any],
                        mount_point: str = '/execution',
                        log_handler: StreamHandler = None):
    register_execution_views(app, cache, services, storage)


###############################################################################
# Internals
###############################################################################
def register_experiment_views(app: Flask, cache: Cache, services: Services,
                              storage: ExperimentStorage):
    pass


def register_execution_views(app: Flask, cache: Cache, services: Services,
                             storage: ExecutionStorage):
    pass


def patch_request(bp: Blueprint, services,
                  storage: Union[ExperimentStorage, ExecutionStorage]):
    @bp.before_request
    def prepare_request():
        request.services = services
        request.storage = storage

        @after_this_request
        def clean_request(response: Response):
            request.services = None
            request.storage = None
            return response



# -*- coding: utf-8 -*-
from uuid import UUID

from flask import abort, Blueprint, jsonify, request
from flask_login import login_required
from marshmallow import ValidationError

from chaosplt_experiment.schemas import upload_execution_schema, \
    execution_schema

__all__ = ["api"]

api = Blueprint("execution", __name__)


@api.route('', methods=['POST'])
@login_required
def new_execution():
    try:
        payload = upload_execution_schema.load(request.json)
    except ValidationError as err:
        return jsonify(err.messages), 422

    journal = payload["journal"]
    experiment_id = payload["experiment_id"]

    has_exp = request.services.experiment.has_experiment_by_id(
        experiment_id=experiment_id)
    if not has_exp:
        return jsonify({
            "message": "Experiment is unknown"
        }), 422

    execution = request.services.execution.create(
        experiment_id, journal)
    return execution_schema.jsonify(execution)


@api.route('<uuid:execution_id>', methods=['GET'])
@login_required
def get_execution(execution_id: UUID):
    execution = request.services.execution.get(execution_id)
    if not execution:
        return abort(404)
    return execution_schema.jsonify(execution)


@api.route('<uuid:execution_id>', methods=['DELETE'])
@login_required
def delete_execution(execution_id: UUID):
    request.services.execution.delete(execution_id)
    return "", 204


@api.route('<uuid:execution_id>/journal', methods=['GET'])
@login_required
def get_journal(execution_id: UUID):
    execution = request.services.execution.get(execution_id)
    if not execution:
        return abort(404)
    return jsonify(execution.payload)



# -*- coding: utf-8 -*-
import glob
import os

import cherrypy
from dotenv import load_dotenv

__all__ = ["load_settings"]


def load_settings(env_path: str):
    """
    Load settings from the environment:

    * if `env_path` is a file, read it
    * if `env_path` is a directory, load, all its `*.env` files
    """
    if os.path.isdir(env_path):
        pattern = os.path.join(env_path, '**', '.env')
        for env_file in glob.iglob(pattern, recursive=True):
            cherrypy.log("Loading: {}".format(env_file))
            load_dotenv(dotenv_path=env_file)
    else:
        cherrypy.log("Loading: {}".format(env_path))
        load_dotenv(dotenv_path=env_path)

    debug = True if os.getenv('CHAOSPLATFORM_DEBUG') else False
    cherrypy.config.update({
        'server.socket_host': os.getenv('SERVER_LISTEN_ADDR'),
        'server.socket_port': int(os.getenv('SERVER_LISTEN_PORT', 8080)),
        'engine.autoreload.on': False,
        'log.screen': debug,
        'log.access_file': '',
        'log.error_file': '',
        'environment': '' if debug else 'production',
        'tools.proxy.on': True,
        'tools.proxy.base': os.getenv('CHERRYPY_PROXY_BASE')
    })

    config = {
        'debug': debug,
        "grpc": {
            "address": os.getenv("GRPC_LISTEN_ADDR"),
        },
        "db": {
            "uri": os.getenv("DATABASE_URI"),
            "debug": debug
        }
    }

    return config



import uuid

from chaosplt_experiment.storage import ExperimentStorage, ExecutionStorage
from chaosplt_experiment.storage.model import Experiment, Execution
from chaosplt_relational_storage.db import orm_session


def test_create_experiment(experiment_storage: ExperimentStorage):
    with orm_session() as session:
        e = Experiment.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment={"title": "a tale of one platform"},
            session=session
        )
        session.commit()
        assert uuid.UUID(hex=e.id.hex) == e.id


def test_load_experiment(experiment_storage: ExperimentStorage):
    with orm_session() as session:
        e = Experiment.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment={"title": "a tale of one platform"},
            session=session
        )
        session.commit()

        fetched_exp = Experiment.load(e.id, session)
        assert e == fetched_exp


def test_load_experiment_by_user(experiment_storage: ExperimentStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        e = Experiment.create(
            user_id=user_id,
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment={"title": "a tale of one platform"},
            session=session
        )
        session.commit()

        fetched_exp = Experiment.load_by_user(user_id, e.id, session)
        assert fetched_exp == e

        fetched_exp = Experiment.load_by_user(uuid.uuid4(), e.id, session)
        assert fetched_exp is None


def test_load_experiment_list_by_user(experiment_storage: ExperimentStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        e = Experiment.create(
            user_id=user_id,
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment={"title": "a tale of one platform"},
            session=session
        )
        session.commit()

        fetched_exps = Experiment.list_all_by_user(user_id, session)
        assert fetched_exps == [e]

        fetched_exps = Experiment.list_all_by_user(uuid.uuid4(), session)
        assert fetched_exps == []


def test_load_experiment_by_workspace(experiment_storage: ExperimentStorage):
    with orm_session() as session:
        workspace_id = uuid.uuid4()

        e = Experiment.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=workspace_id,
            experiment={"title": "a tale of one platform"},
            session=session
        )
        session.commit()

        fetched_exps = Experiment.load_by_workspace(workspace_id, session)
        assert fetched_exps == [e]

        fetched_exp = Experiment.load_by_workspace(uuid.uuid4(), session)
        assert fetched_exp == []


def test_delete_experiment(experiment_storage: ExperimentStorage):
    with orm_session() as session:
        e = Experiment.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment={"title": "a tale of one platform"},
            session=session
        )
        session.commit()

        fetched_exp = Experiment.load(e.id, session)
        assert e == fetched_exp

        Experiment.delete(e.id, session)
        fetched_exp = Experiment.load(e.id, session)
        assert fetched_exp == None


def test_create_execution(experiment_storage: ExperimentStorage,
                          execution_storage: ExecutionStorage):
    with orm_session() as session:
        e = Experiment.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment={"title": "a tale of one platform"},
            session=session
        )
        session.flush()
        e = Execution.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment_id=e.id,
            execution={},
            session=session
        )
        session.commit()
        assert uuid.UUID(hex=e.id.hex) == e.id


def test_load_execution(experiment_storage: ExperimentStorage,
                        execution_storage: ExecutionStorage):
    with orm_session() as session:
        e = Experiment.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment={"title": "a tale of one platform"},
            session=session
        )
        session.flush()
        e = Execution.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment_id=e.id,
            execution={},
            session=session
        )
        session.commit()

        fetched_exp = Execution.load(e.id, session)
        assert e == fetched_exp


def test_load_execution_by_user(experiment_storage: ExperimentStorage,
                                execution_storage: ExecutionStorage):
    with orm_session() as session:
        user_id = uuid.uuid4()

        e = Experiment.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment={"title": "a tale of one platform"},
            session=session
        )
        session.flush()
        e = Execution.create(
            user_id=user_id,
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment_id=e.id,
            execution={},
            session=session
        )
        session.commit()

        fetched_exs = Execution.load_by_user(user_id, session)
        assert fetched_exs == [e]

        fetched_exs = Execution.load_by_user(uuid.uuid4(), session)
        assert fetched_exs == []



def test_delete_execution(experiment_storage: ExperimentStorage,
                          execution_storage: ExecutionStorage):
    with orm_session() as session:
        e = Experiment.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment={"title": "a tale of one platform"},
            session=session
        )
        session.flush()
        e = Execution.create(
            user_id=uuid.uuid4(),
            org_id=uuid.uuid4(),
            workspace_id=uuid.uuid4(),
            experiment_id=e.id,
            execution={},
            session=session
        )
        session.commit()

        fetched_exp = Execution.load(e.id, session)
        assert e == fetched_exp

        Execution.delete(e.id, session)
        fetched_ex = Execution.load(e.id, session)
        assert fetched_ex == None



# -*- coding: utf-8 -*-
import uuid

from chaosplt_relational_storage.db import Base
from sqlalchemy import Column, DateTime, func
from sqlalchemy_utils import UUIDType
from sqlalchemy_utils import JSONType as JSONB

__all__ = ["Discovery"]


class Discovery(Base):  # type: ignore
    __tablename__ = 'experiment_discovery'
    id = Column(
        UUIDType(binary=False), primary_key=True, default=uuid.uuid4)
    account_id = Column(UUIDType(binary=False), nullable=False, index=True)
    received_date = Column(DateTime(), server_default=func.now())
    workspace_id = Column(
        UUIDType(binary=False), nullable=False, index=True)
    payload = Column(JSONB())



import click

from . import __version__
from .server import run_forever
from .settings import load_settings


@click.group()
@click.version_option(version=__version__)
def cli():
    pass


@cli.command()
@click.option('--env-path',
              type=click.Path(exists=True, readable=True, resolve_path=True),
              help='Dot env file or directory path.')
def run(env_path: str):
    """
    Runs the application.
    """
    config = load_settings(env_path)
    run_forever(config)



# -*- coding: utf-8 -*-
from .discovery import Discovery
from .execution import Execution
from .experiment import Experiment
from .init import Init
from .recommendation import Recommendation

__all__ = ["Discovery", "Execution", "Experiment", "Execution", "Init",
           "Recommendation"]



import logging
from logging import StreamHandler
from typing import Any, Dict, Tuple

from chaosplt_grpc import create_grpc_server, start_grpc_server, \
    stop_grpc_server
import cherrypy
from flask import Flask
from flask_caching import Cache
from grpc import Server

from .cache import setup_cache
from .log import http_requests_logger
from .service import initialize_services, shutdown_services, Services
from .storage import ExperimentStorage, ExecutionStorage, \
    initialize_storage, shutdown_storage
from .views.api import create_api, cleanup_api, serve_experiment_api,\
    serve_execution_api
from .views.web import create_app, cleanup_app, serve_experiment_app, \
    serve_execution_app

__all__ = ["initialize_all", "release_all", "run_forever"]
logger = logging.getLogger("chaosplatform")


def initialize_all(config: Dict[str, Any], web_app: Flask = None,
                   api_app: Flask = None, services: Services = None,
                   grpc_server: Server = None, web_cache: Cache = None,
                   api_cache: Cache = None,
                   experiment_web_mount_point: str = "/experiments",
                   execution_web_mount_point: str = "/executions",
                   experiment_api_mount_point: str = "/api/v1/experiments",
                   execution_api_mount_point: str = "/api/v1/executions",
                   access_log_handler: StreamHandler = None) \
                   -> Tuple[
                       Flask, Flask, Services, Server, ExperimentStorage,
                       ExecutionStorage]:
    access_log_handler = access_log_handler or logging.StreamHandler()
    logger.info("Initializing experiment service resources")

    experiment_storage, execution_storage = initialize_storage(config)
    if not services:
        services = Services()
    if not services.experiment:
        services.experiment = experiment_storage
    if not services.execution:
        services.execution = execution_storage

    initialize_services(services, config)

    if not web_app:
        web_app = create_app(config)
        web_cache = setup_cache(web_app)
        wsgiapp = http_requests_logger(web_app, access_log_handler)
        cherrypy.tree.graft(wsgiapp, "/")
    serve_experiment_app(
        web_app, web_cache, services, experiment_storage, config,
        experiment_web_mount_point, log_handler=access_log_handler)
    serve_execution_app(
        web_app, web_cache, services, execution_storage, config,
        execution_web_mount_point, log_handler=access_log_handler)

    if not api_app:
        api_app = create_api(config)
        api_cache = setup_cache(api_app)
        wsgiapp = http_requests_logger(api_app, access_log_handler)
        cherrypy.tree.graft(wsgiapp, "/api/v1")
    serve_experiment_api(
        api_app, api_cache, services, experiment_storage, config,
        experiment_api_mount_point, log_handler=access_log_handler)
    serve_execution_api(
        api_app, api_cache, services, execution_storage, config,
        execution_api_mount_point, log_handler=access_log_handler)

    grpc_server = initialize_grpc(config, grpc_server)

    return (web_app, api_app, services, grpc_server, experiment_storage,
            execution_storage)


def release_all(web_app: Flask, api_app: Flask, services: Services,
                grpc_server: Server, experiment_storage: ExperimentStorage,
                execution_storage: ExecutionStorage):
    logger.info("Releasing experiment service resources")
    if grpc_server:
        logger.info("gRPC server stopping")
        stop_grpc_server(grpc_server)
        logger.info("gRPC server stopped")
    cleanup_app(web_app)
    cleanup_api(api_app)
    shutdown_services(services)
    shutdown_storage(experiment_storage, execution_storage)


def run_forever(config: Dict[str, Any]):
    """
    Run and block until a signal is sent to the process.

    The application, services or gRPC server are all created and initialized
    when the application starts.
    """
    def run_stuff(config: Dict[str, Any]):
        resources = initialize_all(config)
        cherrypy.engine.subscribe(
            'stop', lambda: release_all(*resources),
            priority=20)

    cherrypy.engine.subscribe(
        'start', lambda: run_stuff(config), priority=80)

    cherrypy.engine.signals.subscribe()
    cherrypy.engine.start()
    cherrypy.engine.block()


###############################################################################
# Internals
###############################################################################
def initialize_grpc(config: Dict[str, Any],
                    grpc_server: Server = None) -> Server:
    """
    Initialize the gRPC server

    Create the server if not provided. This called by `initialize_all`.
    """
    if not grpc_server:
        srv_addr = config["grpc"]["address"]
        if srv_addr:
            grpc_server = create_grpc_server(srv_addr)
            start_grpc_server(grpc_server)
            logger.info("gRPC server started on {}".format(srv_addr))

    return grpc_server
